#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# This file is part of Knock subdomain scan (aka knockpy)
#
# Knock is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Knock is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Knock. If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------

import re
import os.path
import urllib2
import string
import time

''' v.0.1
	This module contains the following functions:
	. replace_url_to_link(text)
	. loadfile_wordlist(filename)
	. isfile(filename)
	. touch(filename)
	. downloadfile(fileurl)
	. timestamp()
	. uniqlist(lista)
 '''

def replace_url_to_link(text):
	# Replace url text to link.
	# Credit: https://gist.github.com/guillaumepiot/4539986
	urls = re.compile(r"((https?):((//)|(\\\\))+[\w\d:#@%/;$()~_?\+-=\\\.&]*)", re.MULTILINE|re.UNICODE)
	text = urls.sub(r'<a href="\1" target="_blank">\1</a>', text)
	''' Replace email to mailto '''
	urls = re.compile(r"([\w\-\.]+@(\w[\w\-]+\.)+[\w\-]+)", re.MULTILINE|re.UNICODE)
	text = urls.sub(r'<a href="mailto:\1">\1</a>', text)
	return text

def loadfile_wordlist(filename):
	# Load wordlist from file (rows read per fetched row) and return list.
	filename = open(filename,'r')
	wlist = filename.read().split('\n')
	filename.close
	return filter(None, wlist)
	
def isfile(filename):
	# Verify if file exist.
	return os.path.isfile(filename)

def touch(filename):
	# Create empty file.
	fname = filename
	file = open(fname, 'w')
	file.close()
	
def downloadfile(fileurl):
	# Get content remote file via http(s) and return contenet.
	try: 
		response = urllib2.urlopen(fileurl)
		return response.read()
	except: pass
	
def timestamp():
	return time.time()

def uniqlist(lista):
    ulist = []
    [ulist.append(x) for x in lista if x not in ulist]
    return ulist



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# This file is part of Knock subdomain scan (aka knockpy)
#
# Knock is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Knock is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Knock. If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------

import socket

# set the default timeout on sockets to 5 seconds
if hasattr(socket, 'setdefaulttimeout'):
	socket.setdefaulttimeout(5) # <- here

try:
	import dns.resolver, dns.query, dns.zone
except:
	exit('ImportError: No module named dnspython\ninstall python-dnspython')

def zonetransfer(domain): # Zone Transfer
	ns = []
	try:
		answers = dns.resolver.query(domain,'NS')
	except: exit('zone transfer not found')
	
	for rdata in answers:
		rdata = str(rdata).rstrip('.')
		ns.append(hostbyname(rdata))

	for n in ns:
		zt = []
		try:
			zone = dns.zone.from_xfr(dns.query.xfr(n, domain))
		except: exit('zone transfer not found')
		if zone:
			for name, node in zone.nodes.items():
				rdataset = node.rdatasets
				for record in rdataset:
					name = str(name)
					if name != '@' and name != '*':
						zt.append(name+'.'+domain)
			return zt

def hostbyname(domain):
	try:
		# translate a host name to IPv4 address format
		return socket.gethostbyname(domain)
	except:
		return False

def check(domain):
	found_list = []
	if hostbyname(domain):
		detected = zonetransfer(domain)
		for subdomain in detected:
			ip = hostbyname(subdomain)
			if ip: found_list.append([ip, subdomain])
		return found_list
	else:
		return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# This file is part of Knock subdomain scan (aka knockpy)
#
# Knock is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Knock is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Knock. If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------

import font

def status_wordlist(location, wlist):
	return font.color('ciano')+'\nLoaded '+font.color('bold')+location+font.color('end')\
	+font.color('ciano')+' wordlist with '+font.color('bold')+str(len(wlist))+font.color('end')\
	+font.color('ciano')+' item(s)\n'+font.color('end')
	
def start_scan(domain):
	text = font.color('bold') + 'Getting subdomain for ' + domain + font.color('end') + '\n\n'
	text += 'Ip Address'.ljust(18) + 'Domain Name\n'
	text += '----------'.ljust(18) + '-----------'
	return text

def target(domain):
	text = font.color('bold')+'Target information '+domain+font.color('end')+'\n\n'
	text += 'Ip Address'.ljust(18) + 'Target Name\n'
	text += '----------'.ljust(18) + '-----------'
	return text

def response_code():
	text = 'Code'.ljust(18) + 'Reason\n'
	text += '----------'.ljust(18) + '-----------'
	return text

def response_head():
	text = 'Field'.ljust(18) + 'Value\n'
	text += '----------'.ljust(18) + '-----------'
	return text

def stats_summary():
	return font.color('bold')+'\nSummary\n'+font.color('end')

def start_scan_zt(domain):
	text = font.color('bold') + 'Getting zone transfer for ' + domain + font.color('end') + '\n\n'
	text += 'Ip Address'.ljust(18) + 'Domain Name\n'
	text += '----------'.ljust(18) + '-----------'
	return text



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# This file is part of Knock subdomain scan (aka knockpy)
#
# Knock is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Knock is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Knock. If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------

import socket

# set the default timeout on sockets to 5 seconds
if hasattr(socket, 'setdefaulttimeout'):
	socket.setdefaulttimeout(5) # <- here

def start(target):
	found_alias = []
	found_host = []
	try:
		ipv4 = socket.gethostbyname(target) # Ip Address format
		if ipv4:
			soc = socket.gethostbyname_ex(target)
			# print soc, target # enable for debug
			for alias in soc[1]:
				try:
					# alias -> ip
					ipaddr = socket.gethostbyname(alias)
					found_alias.append([alias, ipaddr])
				except:
					pass

			hostname = soc[0]
			for ipaddr in soc[2]:
				found_host.append([hostname, ipaddr])

		soc.close()
	except: pass
	
	if found_alias or found_host:
		return found_alias, found_host
	else:
		return False

def hostbyip(ip):
	try:
		return socket.gethostbyaddr(ip)[0]
	except:
		return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# This file is part of Knock subdomain scan (aka knockpy)
#
# Knock is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Knock is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Knock. If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------

import utilipy
import font
import os

_ROOT = os.path.abspath(os.path.dirname(__file__))
def get_data(path):
	return os.path.join(_ROOT, 'wordlist', path)

internal_wlist = get_data('wordlist.txt').replace('modules/', '')
remote_wlist = 'https://raw.github.com/guelfoweb/knock/master/wordlist.txt'

def check_status(path_to_wordlist=False):
	if path_to_wordlist:
		local_wlist=path_to_wordlist
	else:
		local_wlist=internal_wlist
	if utilipy.isfile(local_wlist): # local wordlist
		wlist = utilipy.loadfile_wordlist(local_wlist)
		location = 'local'
	else: # remote wordlist
		print font.color('red')+'\nwarning: wordlist not found '+local_wlist+font.color('end')
		res = raw_input('\npress [c] to download remote wordlist or [enter] to exit: ')
		if res != 'c': exit()
		wlist = utilipy.downloadfile(remote_wlist)
		if wlist:
			wlist = wlist.split('\n')
			location = 'remote'
	
	if wlist and location:
		return location, filter(None, wlist)
	else:
		exit('Local wordlist not found\n'+local_wlist+'\n\nRemote wordlist not found or connection error\n'+remote_wlist)

def prepare(domain):
	wordlist = []
	get_info_wordlist = check_status()
	location = get_info_wordlist[0]
	wlist = get_info_wordlist[1]
		
	for sub in wlist:
		wordlist.append(sub+'.'+domain)
		
	return worlist



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# This file is part of Knock subdomain scan (aka knockpy)
#
# Knock is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Knock is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Knock. If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------

def color(style):
	if style == 'yellow':
		return '\033[93m'
	if style == 'red':
		return '\033[91m'
	if style == 'ciano':
		return '\033[36m'
	if style == 'bold':
		return '\033[1m'
	if style == 'end':
		return '\033[0m'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# This file is part of Knock subdomain scan (aka knockpy)
#
# Knock is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Knock is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Knock. If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------

import getheader
import headers
import stats
import target
import utilipy
import wildcard
import wordlist
import subscan
import zonetransfer
import font

def get_header(url, path, method):
	return getheader.req(url, path, method)

def subscan_start(target):
	return subscan.start(target)

def get_info_wordlist(path_to_worlist=False):
	return wordlist.check_status(path_to_worlist)

def prepare_targetlist(domain, wordlist):
	targets = []
	for sub in wordlist:
		targets.append(sub+'.'+domain)
	return targets

def get_target(_target, verbose, test):
	# call from bypass_wildcard(target, wcode)
	return target.get(_target, verbose, test)
		
def stats_general_summary(targetlist):
	return stats.start(found, targetlist)

def bypass_wildcard(target, wcode):
	# call from subdomain_scan()
	if wildcard.bypass(target, wcode):
		return get_target(target, True, False)

def test_wildcard(target):
	return wildcard.test(target)

host_not_found = False
wildcard_detected = False

# Resolve domain
def header_target(domain):
	print headers.target(domain)

def show_resolved(domain, resolve):
	# if [knockpy domain.com] -> resolve is False
	# resolve is True only if use -r option
	
	# return alias and host
	test_host = subscan.start(domain)
	
	# HOST NOT FOUND for:
	# [knockpy domain.com] or [knockpy -r domain.com]
	if not test_host:
		print font.color('red')+'\n: unknown '+domain+font.color('end')
		
		# if [knockpy -r domain.com]
		# bye bye
		if resolve: exit()
		
		# if [knockpy domain.com]
		# prepare query [c] -> continue, [enter] -> exit
		query = 'press '+font.color('bold')+'[c]'\
		+font.color('end')+' to continue to scan or '\
		+font.color('bold')+'[enter]'+font.color('end')\
		+' to exit: '
		
		# prompt
		res = raw_input(query)
		if res != 'c': exit()

		# set values
		host_not_found = True
		# I don't test wildcard, so:
		wildcard_detected = False

		return

	# HOST NOT FOUND for:
	# [knockpy domain.com] or [knockpy -r domain.com]
	
	# get alias and host list
	(alias, host) = test_host[0], test_host[1]

	output = ''

	# if alias exist
	if alias: 
		for name in alias:
			(ipaddr, aliasn) = str(name[1]), str(name[0])
			output += font.color('yellow')+ipaddr.ljust(18)+aliasn+'\n'+font.color('end')

	# host always exists
	len_host = len(host)
	for i in xrange(0, len_host):
		# get hostname by ip
		(ipaddr, hostname) = host[i][1], host[i][0]
		if ipaddr == hostname:
			hostname = subscan.hostbyip(domain)

		output += ipaddr.ljust(18)+hostname+'\n'

	print output

# Code and headers
def get_banner(domain):
	# host_not_found is False by default
	# or set to True by show_resolved(domain, resolve)
	if host_not_found: return
	
	# return [headers]
	# status, reason, headers
	# len = 3
	getinfo_header = getheader.req(domain,'/','HEAD')
	
	# set to global for show_banner(domain)
	global code, reason, header
	if not getinfo_header: 
		(code, header) = False, False
		return
	(code, reason, header) = str(getinfo_header[0]), str(getinfo_header[1]), getinfo_header[2]
	
def header_response_code():
	# host_not_found is False by default
	# or set to True by show_resolved(domain, resolve)
	if host_not_found: return
	print headers.response_code()

def header_response_head():
	# host_not_found is False by default
	# or set to True by show_resolved(domain, resolve)
	if host_not_found: return
	print headers.response_head()

def show_banner(typo):
	# host_not_found is False by default
	# or set to True by show_resolved(domain, resolve)
	if host_not_found: return
	
	# code is global variable from get_banner(domain)
	# return code, reason, header	
	# print code, reason 
	if typo == 'code' and code:	print code.ljust(18)+reason+'\n'
	
	# print headers
	if typo == 'head' and header:
		for head in header:
			# output first chars: 17 fields | 61 values
			print str(head[0])[0:17].ljust(18)+str(head[1])[0:61]

# Wildcard, wordlist, targetlist
def show_wildcard(domain):
	# host_not_found is False by default
	# or set to True by show_resolved(domain, resolve)
	if host_not_found: return
	
	# test wildcard
	global wildcard_detected
	wildcard_detected = False
	
	wildcard_test = wildcard.test(domain)
	if wildcard_test[0]:
		wildcard_detected = True
		
		# set a new value for code
		# from random subdomain response headers
		global wcode
		wcode = str(wildcard_test[0][0])
		print font.color('red')+'\n: wildcard detected: '+wcode+font.color('end')
		

def get_wordlist(domain, path_to_worlist=False):
	# import wordlist
	getinfo_wordlist = get_info_wordlist(path_to_worlist)
	global wordlist
	(location, wordlist) = getinfo_wordlist[0], getinfo_wordlist[1]
	print headers.status_wordlist(location, wordlist)
	
def get_targetlist(domain):
	# prepare subdomain.domain.com
	global targetlist
	targetlist = prepare_targetlist(domain, wordlist)

# Start
def header_start_scan(domain):
	print headers.start_scan(domain)

def subdomain_scan():
	for target in targetlist:
		if wildcard_detected:
			result = bypass_wildcard(target, wcode)
		else:
			result = get_target(target, False, False)

		if result:
			print result

# Statistics
def header_stats_summary():
	return headers.stats_summary()

def report():
	print target.get_report(targetlist)

# Save result in csv
def save_in_csv(domain):
	print target.save_csv(domain)

# Zone transfer
def getzone(domain):
	detected = zonetransfer.check(domain)
	if not detected: exit('zone transfer not found')
	print headers.start_scan_zt(domain)
	for item in detected:
		(ip, subdomain) = str(item[0]), str(item[1])
		print ip.ljust(18)+subdomain




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# This file is part of Knock subdomain scan (aka knockpy)
#
# Knock is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Knock is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Knock. If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------

import httplib

agent = "Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:35.0) Gecko/20100101 Firefox/35.0"

def req(url, path, method):
	try:
		conn = httplib.HTTPConnection(url)
		conn.putrequest(method, path)
		conn.putheader("User-Agent", agent)
		'''
		#conn.putheader("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8")
		#conn.putheader("Connection", "keep-alive")

		# Test for XST
		conn.putheader('Via', '<script>alert(0)</script>')
		conn.request(method, path)
		'''
		conn.endheaders()
		res = conn.getresponse()
		conn.close()
		return res.status, res.reason, res.getheaders()
	except:
		return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Knock subdomain scan (aka knockpy)
#
# Knock is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Knock is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Knock. If not, see <http://www.gnu.org/licenses/>.

from modules import core
import argparse

__version__='3.0'
__description__='''\
  ___________________________________________
  
  knock subdomain scan (aka knockpy) | v.'''+__version__+'''
  Author: Gianni 'guelfoweb' Amato
  Github: https://github.com/guelfoweb/knock
  ___________________________________________
'''

def getinfo(domain, resolve):
	# Resolve domain
	core.header_target(domain)
	# if [knockpy domain.com] -> resolve is False
	# if [knockpy -r domain.com] -> resolve is True
	core.show_resolved(domain,resolve)

	# Status code
	core.header_response_code()
	core.get_banner(domain)
	core.show_banner('code')

	# Headers
	core.header_response_head()
	core.show_banner('head')

	# Wildcard
	core.show_wildcard(domain)

def get_wordlist_targetlist(domain, path_to_worlist=False):
	# Wordlist, targetlist
	core.get_wordlist(domain, path_to_worlist)
	core.get_targetlist(domain)

def start(domain):
	# Start
	core.header_start_scan(domain)
	core.subdomain_scan()

def statistics():
	# Statistics
	core.header_stats_summary()
	core.report()

def savescan(domain):
	# Save result
	core.save_in_csv(domain)

def getzone(domain):
	core.getzone(domain)
	
def main():
	parser = argparse.ArgumentParser(
		version=__version__,
		formatter_class=argparse.RawTextHelpFormatter,
		prog='knockpy',
		description=__description__,
		epilog = '''\
EXAMPLE:

subdomain scan with internal wordlist
  knockpy domain.com

resolve domain name and get response headers
  knockpy -r domain.com

check zone transfer for domain name
  knockpy -z domain.com

The ALIAS name is marked in yellow''')

	parser.add_argument('domain', help='specific target domain, like domain.com')

	parser.add_argument('-w', help='specific path to wordlist file',
						nargs=1, dest='wordlist', required=False)

	parser.add_argument('-r', '--resolve', help='resolve ip or domain name',
						action='store_true', required=False)

	parser.add_argument('-z', '--zone', help='check for zone transfer',
						action='store_true', required=False)

	args = parser.parse_args()

	# args strings
	domain = args.domain
	wlist = args.wordlist
	if wlist: wlist = wlist[0]

	# args True or False
	resolve = args.resolve
	zone = args.zone

	# [knockpy -r domain.com]
	if domain and resolve and not zone:
		# resolve = True
		getinfo(domain, resolve)

	# [knockpy -z domain.com]
	elif domain and zone and not resolve:
		getzone(domain)

	# [knockpy domain.com]
	elif domain and not resolve and not zone:
		# resolve = False
		getinfo(domain, resolve)
		
		if wlist:
			get_wordlist_targetlist(domain, wlist)
		else:
			# get_wordlist_targetlist(domain,path_to_worlist=False)
			# no wlist
			get_wordlist_targetlist(domain)
			
		start(domain)
		statistics()
		savescan(domain)

	else:
		exit('error arguments: use knockpy -h to help')

if __name__ == '__main__':
	main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# This file is part of Knock subdomain scan (aka knockpy)
#
# Knock is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Knock is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Knock. If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------

import getheader
import random
import socket

def bypass(target, wcode):
	# beta feature
	text = ''
	header = getheader.req(target,'/','HEAD')
	# bypass status code -> header[0] = 301 or 302
	if header and str(header[0]) != wcode: return True
	else: return False

def test(target):
	# call from show_wildcard(domain)
	rndString = rnd('abcdefghijklmnopqrstuvwxyz')
	rndSubdomain = str(rndString)+'.'+target
	try:
		host  = socket.gethostbyname(rndSubdomain)
		if host:
			httpreq = getheader.req(rndSubdomain,'/','HEAD')
			return httpreq, True
	except:
		return False, False

def rnd(alphabet): # random string
	# alphabet  = 'abcdefghijklmnopqrstuvwxyz'
	min       = 5
	max       = 15
	total     = 2
	rndstring = ''
	for count in xrange(1,total):
		for x in random.sample(alphabet,random.randint(min,max)):
			rndstring+=x
	return rndstring



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# This file is part of Knock subdomain scan (aka knockpy)
#
# Knock is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Knock is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Knock. If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------

from setuptools import setup
from codecs import open  # To use a consistent encoding
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the relevant file
with open(path.join(here, 'CHANGELOG.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='knockpy',
    version='3.0',

    description='Knock is a python tool designed to enumerate subdomains on a target domain through a wordlist.',
    long_description=long_description,
    url='https://github.com/guelfoweb/knock',

    author='Gianni \'guelfoweb\' Amato',
    author_email='guelfoweb@gmail.com',

    license='GNU',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: GNU General Public License (GPL)',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],

    keywords='knock sudbomain scan',

    packages=["knockpy", "knockpy.modules"],
    package_data={'knockpy': ['wordlist/wordlist.txt']},
	install_requires = ['dnspython>=1.3.5'],

    entry_points={
        'console_scripts': [
            'knockpy=knockpy.knockpy:main',
        ],
    },

)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# This file is part of Knock subdomain scan (aka knockpy)
#
# Knock is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Knock is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Knock. If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------

import socket
import font

def cogito(found, targetlist):
	# print found # enable for debug
	ipaddr = []
	subdomain = []
	subdomain_in_wlist = []
	subdomain_for_ip = []

	len_found = len(found)

	# ip address discovered
	for i in xrange(0, len_found):
		ipaddr.append(found[i][1])
	uniq_ipaddr = set(ipaddr)

	# subdomain discovered
	for i in xrange(0, len_found):
		subdomain.append(found[i][2])
	uniq_subdomain = set(subdomain)
	
	# subdomain in wordlist
	for subdomain in uniq_subdomain:
		if subdomain in targetlist:
			subdomain_in_wlist.append(subdomain)
	uniq_wlist = set(subdomain_in_wlist)

	report = '\nFound '+font.color('bold')+str(len(uniq_subdomain))\
	+font.color('end')+' subdomain(s) in '+font.color('bold')+str(len(uniq_ipaddr))+font.color('end')+' host(s).\n'\
	+font.color('bold')+str(len(uniq_wlist))+font.color('end')+'/'+font.color('bold')+str(len(uniq_subdomain))+font.color('end')\
	+' subdomain(s) are in wordlist.'
	return report



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# This file is part of Knock subdomain scan (aka knockpy)
#
# Knock is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Knock is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Knock. If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------

import font
import subscan
import utilipy
import stats

global found
found = []
def get(target, verbose, test):
	text = ''
	result = subscan.start(target)
	if result: (alias, host) = result[0], result[1]
	else: return
		
	# Detect alias
	for item in alias:
		found.append([target, item[1], item[0], 'alias'])
		text += font.color('yellow') + str(item[1]).ljust(18) + str(item[0]) + font.color('end') + '\n'
		# Test wildcard to detect host
		if verbose and not test:
			for item in host:
				found.append([target, item[1], item[0], 'host'])
				text += str(item[1]).ljust(18) + str(item[0]) + '\n'

	# Test subdomain to detect host
	if not verbose and not test:
		for item in host:
			found.append([target, item[1], item[0], 'host'])
			text += str(item[1]).ljust(18) + str(item[0]) + '\n'

	# Test root domain to detect host
	if test:
		for item in host:
			found.append([target, item[1], item[0], 'host'])
			text += str(item[1]).ljust(18) + str(item[0]) + '\n'

	return text.rstrip()

def save_csv(domain):
	if not found: exit()
	timestamp = utilipy.timestamp()
	filename = domain.replace('.', '_')+'_'+str(timestamp)+'.csv'
	try:
		utilipy.touch(filename)
		with open(filename, 'a') as f:
			f.write('target,ip address,domain name,type\n')
			for row in found:
				f.write(row[0]+','+row[1]+','+row[2]+','+row[3]+'\n')
		f.close()
		return '\nOutput saved in CSV format: '+filename
	except: return '\nCannot write csv file: '+filename
	
def get_report(targetlist):
	return stats.cogito(found, targetlist)
